<?php
/*
Plugin Name: MU Custom Reset Password Handler (v3.3)
Description: Fixes the reset URL to always point to the current subsite instead of the network root.
Version: 3.3
Author: ChatGPT
*/

error_log('📌 MU Plugin loaded');

// 🔑 Save the raw reset key separately on main site
add_filter( 'retrieve_password_key', function( $key, $user_login ) {
    $user = get_user_by( 'login', $user_login );
    if ( $user ) {
        switch_to_blog(1);
        update_user_meta( $user->ID, 'reset_raw_key', $key );
        restore_current_blog();
        error_log('💾 נשמר reset_raw_key=' . $key);
    }
    return $key;
}, 10, 2 );

// ✉️ Customize reset email with subsite URL (including language path like /he/)
add_filter( 'retrieve_password_message', function( $message, $key, $user_login, $user_data ) {
    $host = $_SERVER['HTTP_HOST'] ?? parse_url(home_url(), PHP_URL_HOST);
    $scheme = is_ssl() ? 'https://' : 'http://';
    $path = parse_url(home_url(), PHP_URL_PATH) ?? '';
    $base_url = $scheme . $host . rtrim($path, '/') . '/reset-password/';

    $custom_url = add_query_arg([
        'key'   => rawurlencode($key),
        'login' => rawurlencode($user_login),
    ], $base_url);

    error_log('📤 מייל איפוס נשלח ל-' . $user_login . ' עם הקישור: ' . $custom_url);

    return sprintf(
        __("שלום,

כדי לאפס את הסיסמה שלך, לחץ על הקישור הבא:

%s

אם לא ביקשת איפוס, אין צורך לעשות דבר.", 'textdomain'),
        $custom_url
    );
}, 10, 4 );

// 📨 Handle reset form logic
add_action('elementor_pro/forms/new_record', function( $record, $handler ) {
    global $wpdb;

    $form_name = $record->get_form_settings('form_name');
    error_log('📄 Form name: ' . $form_name);
    if ('custom_reset_password' !== $form_name) return;

    $raw_fields = $record->get('fields');
    $fields = [];
    foreach ( $raw_fields as $field ) {
        $fields[ $field['id'] ] = $field['value'];
    }

    $key   = sanitize_text_field($fields['reset_key'] ?? '');
    $login = sanitize_user($fields['reset_login'] ?? '');

    error_log('🔑 key=' . $key . ' | login=' . $login);
    if ( empty($key) || empty($login) ) {
        $handler->add_error('', 'קישור איפוס שגוי');
        error_log('❌ key או login חסרים');
        return;
    }

    $user = get_user_by('login', $login);
    if (!$user) {
        $handler->add_error('', 'משתמש לא נמצא');
        error_log('❌ משתמש לא נמצא');
        return;
    }

    $row = $wpdb->get_row(
        $wpdb->prepare("SELECT user_activation_key FROM {$wpdb->users} WHERE ID = %d", $user->ID)
    );
    $stored_key = $row ? $row->user_activation_key : '';
    error_log('🔐 stored_key=' . $stored_key);

    $parts = explode(':', $stored_key);
    $hash = $parts[1] ?? '';

    switch_to_blog(1);
    $saved_raw_key = get_user_meta($user->ID, 'reset_raw_key', true);
    restore_current_blog();

    error_log('🔁 raw_key from meta=' . $saved_raw_key);

    if (
        empty($hash) ||
        ( $saved_raw_key !== $key && !wp_check_password($key, $hash, $user->ID) )
    ) {
        $handler->add_error('', 'קישור האיפוס לא תקף או פג תוקף');
        error_log('❌ מפתח לא תואם לפי hash או raw_key');
        return;
    }

    $pass1 = sanitize_text_field($fields['new_password'] ?? '');
    $pass2 = sanitize_text_field($fields['confirm_password'] ?? '');

    if (empty($pass1) || empty($pass2)) {
        $handler->add_error('new_password', 'נא להזין סיסמה חדשה ואישור');
        error_log('❗ שדות הסיסמה ריקים');
        return;
    }

    if ($pass1 !== $pass2) {
        $handler->add_error('confirm_password', 'הסיסמאות אינן תואמות');
        error_log('❗ סיסמאות לא תואמות');
        return;
    }

    reset_password($user, $pass1);
    error_log('✅ הסיסמה אופסה בהצלחה עבור: ' . $login);
}, 10, 2 );
